/*
 * Miscellanea, helper classes for the EDG and e-Knife projects
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package misc;

import java.io.File;
import java.net.URLDecoder;

public abstract class Config
{
	/********************************************************************************************************************************/
	/************************************************************ OBJECT ************************************************************/
	/********************************************************************************************************************************/
	protected final String programPath;
	protected final String binariesPath;
	protected final String sourcesPath;
	protected final String rootPath;
	protected final String librariesPath;
	protected final String configurationPath;
	protected final String miscellaneaPath;
	protected final String temporaryPath;

	/****************************************************************/
	/************************* Constructor **************************/
	/****************************************************************/
	public Config()
	{
		final String projectPath = Misc.getProjectPath(this.getClass());
		this.programPath = this.calculateProgramPath(projectPath);
		this.binariesPath = this.programPath + "bin" + File.separator;
		this.sourcesPath = this.programPath + "src" + File.separator;

		final boolean isJar = projectPath.endsWith(".jar!" + File.separator);
		this.rootPath = isJar ? this.programPath + "Resources" + File.separator : this.programPath;
		this.librariesPath = this.rootPath + "lib" + File.separator;
		this.configurationPath = this.rootPath + "cfg" + File.separator;
		this.miscellaneaPath = this.rootPath + "misc" + File.separator;
		this.temporaryPath = this.rootPath + "tmp" + File.separator;
	}
	private String calculateProgramPath(String projectPath)
	{
		String programPath = projectPath;

		final String jarEnding = "!" + File.separator;
		final boolean isJar = projectPath.endsWith(".jar" + jarEnding);
		programPath = isJar ? programPath.substring(0, programPath.length() - jarEnding.length()) : programPath;
		programPath = isJar ? programPath.substring(0, programPath.lastIndexOf(File.separator)) : programPath;
		final boolean finishWithBin = programPath.endsWith(File.separator + "bin") || programPath.endsWith(File.separator + "bin" + File.separator);
		programPath = finishWithBin ? programPath.substring(0, programPath.lastIndexOf("bin")) : programPath;
		final boolean finishWithLib = programPath.endsWith(File.separator + "lib") || programPath.endsWith(File.separator + "lib" + File.separator);
		programPath = finishWithLib ? programPath.substring(0, programPath.lastIndexOf("lib")) : programPath;
		final boolean finishWithResources = programPath.endsWith(File.separator + "Resources") || programPath.endsWith(File.separator + "Resources" + File.separator);
		programPath = finishWithResources ? programPath.substring(0, programPath.lastIndexOf("Resources")) : programPath;

		programPath = programPath.endsWith(File.separator) ? programPath : programPath + File.separator;

		try
		{
			programPath = URLDecoder.decode(programPath, "UTF-8");
		}
		catch (Exception e)
		{

		}

		return programPath;
	}

	/****************************************************************/
	/**************************** Paths *****************************/
	/****************************************************************/
	public String getProgramPath()
	{
		return this.programPath;
	}
	public String getBinariesPath()
	{
		return this.binariesPath;
	}
	public String getSourcesPath()
	{
		return this.sourcesPath;
	}
	public String getLibrariesPath()
	{
		return this.librariesPath;
	}
	public String getConfigurationPath()
	{
		return this.configurationPath;
	}

	public String getMiscellaneaPath()
	{
		return this.miscellaneaPath;
	}

	public String getTemporaryPath()
	{
		return this.temporaryPath;
	}

	/****************************************************************/
	/**************************** Files *****************************/
	/****************************************************************/
	public File getProgramFile()
	{
		return new File(this.getProgramPath());
	}

	public File getBinariesFile()
	{
		return new File(this.getBinariesPath());
	}
	public File getSourcesFile()
	{
		return new File(this.getSourcesPath());
	}
	public File getLibrariesFile()
	{
		return new File(this.getLibrariesPath());
	}
	public File getConfigurationFile()
	{
		return new File(this.getConfigurationPath());
	}
	public File getMiscellaneaFile()
	{
		return new File(this.getMiscellaneaPath());
	}
	public File getTemporaryFile()
	{
		return new File(this.getTemporaryPath());
	}
}
